﻿Imports Tsql
Imports System.Data.Odbc
Imports System.Collections.ObjectModel

''' -----------------------------------------------------------------------------
''' <summary>
''' Provides functions for building and manipulating a database group criteria 
''' string collection.
''' </summary>
''' <remarks>Provides functions for generating a human readable group criteria 
''' collection string as well as an SQL group criteria collection string.
''' </remarks>
''' -----------------------------------------------------------------------------
<Serializable()> _
Public Class GroupByCollection
    Inherits Collection(Of GroupBy)

#Region "Constructors"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a new <c>GroupByCollection</c> object.
    ''' </summary>
    ''' <remarks>
    ''' Creates a new empty <c>GroupByCollection</c> object.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub New()
        MyBase.New()
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a new <c>GroupByCollection</c> object from a group criteria 
    ''' string.
    ''' </summary>
    ''' <param name="strGroupBy">A group criteria string in the format
    ''' <c>{ <i>columnname</i> <b>(</b> [ <b>ascending</b> | <b>descending</b> ] 
    ''' <b>)</b> }, ...</c>.</param>
    ''' <remarks>
    ''' Creates a new <c>GroupByCollection</c> object and initialises it with
    ''' the group criteria string specified in the <paramref name="strGroupBy"/>
    ''' parameter.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub New(ByVal strGroupBy As String)
        MyBase.New()
        Me.Parse(strGroupBy)
    End Sub

#End Region

#Region "Properties"

#End Region

#Region "Methods"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Initialises a <c>GroupByCollection</c> object with an SQL group criteria 
    ''' string.
    ''' </summary>
    ''' <param name="strGroupBy">An SQL group criteria string in the format
    ''' <c>{ <i>columnname</i> [ <b>ASC</b> | <b>DESC</b> ] }, ...</c>.</param>
    ''' <remarks>Initialises the <c>GroupByCollection</c> object with the SQL
    ''' group criteria string specified in the <paramref name="strGroupBy"/> 
    ''' parameter.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub FromSql(ByVal strGroupBy As String)
        Me.Clear()
        If (strGroupBy Is Nothing) Then
            Exit Sub
        End If

        If (strGroupBy.Length = 0) Then
            Exit Sub
        End If

        Dim strParts() As String = strGroupBy.Split(Keyword.FieldDelimiter)
        Dim nPart As Integer
        For nPart = 0 To strParts.Length - 1
            Dim scGroupBy As New GroupBy
            scGroupBy.FromSql(strParts(nPart))
            Add(scGroupBy)
        Next
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="strGroupBy"></param>
    ''' <remarks></remarks>
    Public Sub Parse(ByVal strGroupBy As String)
        If (strGroupBy Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (strGroupBy.Length = 0) Then
            Exit Sub
        End If

        Dim strParts() As String = strGroupBy.Split(Keyword.FieldDelimiter)
        Dim nPart As Integer
        For nPart = 0 To strParts.Length - 1

            Dim scGroupBy As New GroupBy(strParts(nPart).Trim())
            Me.Add(scGroupBy)
        Next
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a human readable group criteria string from a 
    ''' <c>GroupByCollection</c>.
    ''' </summary>
    ''' <returns>A group criteria string.</returns>
    ''' <remarks>
    ''' <p>If the <c>GroupByCollection</c> is empty, an <c>Empty</c> string is 
    ''' returned.</p>
    ''' <p>The returned string is in the format <c>[ <i>columnname</i> <b>(</b> [ 
    ''' <b>ascending</b> | <b>descending</b> ] <b>)</b> ],... </c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overrides Function ToString() As String

        Dim strCriteria As String = ""
        Dim strGroupBy As String = ""
        Dim scGroupBy As GroupBy

        For Each scGroupBy In Me
            strCriteria = scGroupBy.ToString
            If Not (strCriteria.Length = 0) Then
                If Not (strGroupBy.Length = 0) Then
                    strGroupBy = String.Concat(strGroupBy, Keyword.FieldDelimiter)
                End If
                strGroupBy = String.Concat(strGroupBy, strCriteria)
            End If
        Next
        Return strGroupBy

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL group criteria string from a <c>GroupByCollection</c>.
    ''' </summary>
    ''' <returns>An SQL group criteria string.</returns>
    ''' <remarks>
    ''' <p>If the <c>GroupByCollection</c> is empty, an <c>Empty</c> string is 
    ''' returned.</p>
    ''' <p>The returned string is in the format <c>[ <i>columnname</i> [ <b>ASC</b> | 
    ''' <b>DESC</b> ] ],... </c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Function ToSql() As String

        Dim strCriteria As String
        Dim strSql As String = Nothing
        Dim scGroupBy As GroupBy

        If (Me.Count = 0) Then
            Return String.Empty
        End If

        For Each scGroupBy In Me
            strCriteria = scGroupBy.ToSql
            If Not (strCriteria.Length = 0) Then
                If Not (strSql Is Nothing) Then
                    strSql = String.Concat(strSql, Keyword.FieldDelimiter)
                End If
                strSql = String.Concat(strSql, strCriteria)
            End If
        Next
        Return strSql

    End Function

    Public Function ToGroupBySql() As String

        Dim strCriteria As String
        Dim strSql As String = Nothing
        Dim scGroupBy As GroupBy

        If (Me.Count = 0) Then
            Return String.Empty
        End If

        For Each scGroupBy In Me
            strCriteria = Field.GetQuotedFieldName(scGroupBy.ColumnName)
            If Not (strCriteria.Length = 0) Then
                If Not (strSql Is Nothing) Then
                    strSql = String.Concat(strSql, Keyword.FieldDelimiter)
                End If
                strSql = String.Concat(strSql, strCriteria)
            End If
        Next
        Return strSql

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL group criteria string from a <c>GroupByCollection</c>.
    ''' </summary>
    ''' <param name="cnn">An open connection to the database.</param>
    ''' <param name="strTableName">The name of the database table the columns are 
    ''' in.</param>
    ''' <returns>An SQL group criteria string.</returns>
    ''' <remarks>
    ''' <p>If the <c>GroupByCollection</c> is empty, an <c>Empty</c> string is 
    ''' returned.</p>
    ''' <p>The returned string is in the format <c>[ <i>columnname</i> [ <b>ASC</b> | 
    ''' <b>DESC</b> ] ],... </c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Function ToSql(ByVal cnn As OdbcConnection, ByVal strTableName As String) As String

        If IsNothing(cnn) Then
            Throw New ArgumentNullException("cnn")
        End If
        If IsNothing(strTableName) Then
            Throw New ArgumentNullException("strTableName")
        End If

        Dim strCriteria As String
        Dim strSql As String = Nothing
        Dim scGroupBy As GroupBy

        If (Me.Count = 0) Then
            Return String.Empty
        End If

        For Each scGroupBy In Me
            strCriteria = scGroupBy.ToSql(cnn, strTableName)
            If Not (strCriteria.Length = 0) Then
                If Not (strSql Is Nothing) Then
                    strSql = String.Concat(strSql, Keyword.FieldDelimiter)
                End If
                strSql = String.Concat(strSql, strCriteria)
            End If
        Next
        Return strSql

    End Function
#End Region
End Class
